import os
import tensorflow as tf
from keras.preprocessing.image import ImageDataGenerator
import matplotlib.pyplot as plt

print("Num GPUs Available: ", len(tf.config.experimental.list_physical_devices('GPU')))

# 训练图片路径
TRAINING_DIR = '../rps/'
VALIDATION_DIR = '../rps-test-set/'
print('TRAINING_DIR: ' + ' '.join(os.listdir(TRAINING_DIR)))
print('VALIDATION_DIR: ' + ' '.join(os.listdir(VALIDATION_DIR)))

# 图像预处理
training_datagen = ImageDataGenerator(
    rescale=1. / 255,  # 重缩放因子。默认为 None。如果是 None 或 0，不进行缩放，否则将数据乘以所提供的值（在应用任何其他转换之前）。
    rotation_range=40,  # 整数。随机旋转的度数范围。
    width_shift_range=0.2,
    height_shift_range=0.2,
    shear_range=0.2,  # 浮点数。剪切强度（以弧度逆时针方向剪切角度）。
    zoom_range=0.2,  # 随机缩放范围。
    horizontal_flip=True,  # 布尔值。随机水平翻转。
    fill_mode='nearest',  # 默认为 'nearest'。输入边界以外的点根据给定的模式填充。
)
train_generator = training_datagen.flow_from_directory(
    TRAINING_DIR,
    target_size=(150, 150),  # 所有的图像将被调整到的尺寸。
    class_mode='categorical',  # 决定返回的标签数组的类型；'categorical' 将是 2D one-hot 编码标签。
    batch_size=126,  # 一批数据的大小（默认 32）。
)

validation_datagen = ImageDataGenerator(rescale=1. / 255)
validation_generator = validation_datagen.flow_from_directory(
    VALIDATION_DIR,
    target_size=(150, 150),
    class_mode='categorical',
    batch_size=126,
)
# 图像预处理 end

# 定义神经网络
model = tf.keras.models.Sequential([
    # Note the input shape is the desired size of the image 150x150 with 3 bytes color
    # This is the first convolution
    tf.keras.layers.Conv2D(64, (3, 3), activation='relu', input_shape=(150, 150, 3)),
    tf.keras.layers.MaxPooling2D(2, 2),  # 最大值池化。
    # The second convolution
    tf.keras.layers.Conv2D(64, (3, 3), activation='relu'),  # 第二层神经网络。
    tf.keras.layers.MaxPooling2D(2, 2),
    # The third convolution
    tf.keras.layers.Conv2D(128, (3, 3), activation='relu'),
    tf.keras.layers.MaxPooling2D(2, 2),
    # The fourth convolution
    tf.keras.layers.Conv2D(128, (3, 3), activation='relu'),
    tf.keras.layers.MaxPooling2D(2, 2),
    # Flatten the results to feed into a DNN
    tf.keras.layers.Flatten(),
    tf.keras.layers.Dropout(0.5),
    # 512 neuron hidden layer
    tf.keras.layers.Dense(512, activation='relu'),
    # 输出有 3 个神经元，分别输出 石头 剪刀 布 的概率
    tf.keras.layers.Dense(3, activation='softmax')
])
model.summary()  # 输出神经网络。
model.compile(
    loss='categorical_crossentropy',  # 损失函数名。
    optimizer='rmsprop',  # 优化器名。
    metrics=['accuracy']  # 在训练和测试期间的模型评估标准。
)
# 定义神经网络 end

history = model.fit(
    train_generator,
    epochs=25,  # 训练模型迭代轮次。
    steps_per_epoch=20,  # 在声明一个轮次完成并开始下一个轮次之前的总步数。
    validation_data=validation_generator,  # 用来评估损失。
    verbose=1,  # 日志显示模式。 0 = 安静模式, 1 = 进度条, 2 = 每轮一行。
    validation_steps=3,  # 只有在指定了 steps_per_epoch 时才有用。停止前要验证的总步数。
)

# 生成对比训练和测试的图
acc = history.history['accuracy']
val_acc = history.history['val_accuracy']
loss = history.history['loss']
val_loss = history.history['val_loss']

epochs = range(len(acc))

plt.plot(epochs, acc, 'r', label='Training accuracy')
plt.plot(epochs, val_acc, 'b', label='Validation accuracy')
plt.plot(epochs, loss, 'g', label='Training loss')
plt.plot(epochs, val_loss, 'm', label='Validation loss')
plt.title('Training and validation accuracy')
plt.legend(loc=0)
plt.figure()
# 老是白图
# plt.savefig('../verification_result.jpg')
plt.show()

model.save('rps.h5')
